(function (Drupal, $) {
  Drupal.behaviors.dreamPermissions = {
    attach: function (context, settings) {
      $('body').once('dream-permissions', function () {
        var $buttonFetch = $('<button>Fetch permissions</button>');
        var $buttonSave = $('<button>Save permissions</button>');
        var $actions = $('.dream-permissions--form .form-actions');
        var $checks = $('.dream-permissions--checks');
        var token = '';
        var originalPermissions = {};

        $actions.append($buttonFetch);
        $actions.append($buttonSave);
        $('.dream-permissions--form #edit-submit').remove();

        $('.dream-permissions--form .dream-permissions--filter').bind('input', function (event) {
          var $filter = $(this);
          var search = $filter.val().toLowerCase();
          $filter.closest('.fieldset-wrapper').find('.form-type-checkbox').each(function () {
            if (this.innerText.toLowerCase().indexOf(search) != -1) {
              this.classList.remove('unmatched');
            }
            else {
              this.classList.add('unmatched');
            }
          });
        });

        $buttonSave.bind('click', function (event) {
          event.preventDefault();
          var data = {};
          $checks.find('input[type="checkbox"]').map(function () {
            // Split name in role and permission.
            var parts = this.name.match(/(\d*)\[(.*)\]/);
            if (typeof data[parts[1]] === "undefined") {
              data[parts[1]] = {};
            }
            data[parts[1]][parts[2]] = this.checked ? '1' : '0';
          });
          console.log(data);

          // Issue a post request.
          $.post(Drupal.settings.basePath + Drupal.settings.pathPrefix + 'admin/people/dream_permissions/save', {
            'token': token,
            'originalPermissions': originalPermissions,
            'permissions': data
          })
        });

        $buttonFetch.bind('click', function (event) {
          event.preventDefault();
          var selectedRoles = $('.dream-permissions--role-names input:checked').map(function () {
            return this.value;
          }).get().join(",");
          var selectedModules = $('.dream-permissions--module-names input:checked').map(function () {
            return this.value;
          }).get().join(",");

          if (selectedRoles.length > 0 && selectedModules.length > 0) {
            $.get(Drupal.settings.basePath + Drupal.settings.pathPrefix + 'admin/people/dream_permissions/fetch/' + selectedModules + '/' + selectedRoles, function (data) {
              console.log(data);

              // Store the token.
              token = data.token;

              // Collapse fieldsets.
              $('.dream-permissions--form fieldset').addClass('collapsed');

              // Clean output.
              $checks.empty();
              $table = $('<table></table>');

              // Add header row.
              var roles = data.roles;
              var $row = $('<tr></tr>');
              $row.append('<th class="label">' + Drupal.t('Permission') + '</th>');
              for (var role in roles) {
                $row.append('<th class="label">' + roles[role] + '</th>');
              }
              $table.append($row);

              // Add checkbox rows.
              var permissions = data.permissions;
              originalPermissions = permissions;
              var permissionsNames = data.permissionsNames;

              for (var permissionName in permissions['2']) {
                var $row = $('<tr></tr>');
                $row.append('<td class="label">' + permissionsNames[permissionName] + '</td>');
                for (var permission in permissions) {
                  if (permissions[permission][permissionName] === '1') {
                    $row.append('<td class="item"><input type="checkbox" name="' + permission + '[' + permissionName + ']" value="1" checked="checked"></td>');
                  }
                  else {
                    $row.append('<td class="item"><input type="checkbox" name="' + permission + '[' + permissionName + ']" value="1"></td>');
                  }
                }
                $table.append($row);
              }

              $checks.append($table);
            });
          }
        });
      });
    }
  };
})(Drupal, jQuery);
